%{?mingw_package_header}

# The tests take ages to run and require Wine.
%global run_tests 0
%global prerelease_date 20110511

Name:           mingw-pthreads
Version:        2.8.0
Release:        22.%{prerelease_date}cvs%{?dist}
Summary:        MinGW pthread library
License:        LGPLv2+
Group:          Development/Libraries
URL:            http://sourceware.org/pthreads-win32/
# cvs -d :pserver:anoncvs@sourceware.org:/cvs/pthreads-win32 login
# cvs -d :pserver:anoncvs@sourceware.org:/cvs/pthreads-win32 export -D 2011-05-11 -d pthreads-w32-20110511 pthreads
# tar cjf pthreads-w32-20110511.tar.bz2 pthreads-w32-20110511
Source0:        pthreads-w32-%{prerelease_date}.tar.bz2

BuildArch:      noarch

Patch0:         mingw32-pthreads-2.8.0-20110511-use-wine-for-tests.patch
Patch1:         mingw32-pthreads-2.8.0-20110511-no-failing-tests.patch
Patch2:         mingw32-pthreads-flags.patch

BuildRequires:  mingw32-filesystem >= 95
BuildRequires:  mingw64-filesystem >= 95
BuildRequires:  mingw32-gcc
BuildRequires:  mingw64-gcc
BuildRequires:  mingw32-gcc-c++
BuildRequires:  mingw64-gcc-c++
BuildRequires:  mingw32-binutils
BuildRequires:  mingw64-binutils
BuildRequires:  dos2unix

%if %{run_tests}
BuildRequires:  wine
%endif


%description
The POSIX 1003.1-2001 standard defines an application programming
interface (API) for writing multithreaded applications. This interface
is known more commonly as pthreads. A good number of modern operating
systems include a threading library of some kind: Solaris (UI)
threads, Win32 threads, DCE threads, DECthreads, or any of the draft
revisions of the pthreads standard. The trend is that most of these
systems are slowly adopting the pthreads standard API, with
application developers following suit to reduce porting woes.

Win32 does not, and is unlikely to ever, support pthreads
natively. This project seeks to provide a freely available and
high-quality solution to this problem.

# Win32
%package -n mingw32-pthreads
Summary:        MinGW pthread library for the win32 target

# Fix upgrade path when upgrading from the testing repo to F17
Obsoletes:      mingw32-winpthreads < 0-0.2.20120224
Provides:       mingw32-winpthreads = 0-0.2.20120224

%description -n mingw32-pthreads
The POSIX 1003.1-2001 standard defines an application programming
interface (API) for writing multithreaded applications. This interface
is known more commonly as pthreads. A good number of modern operating
systems include a threading library of some kind: Solaris (UI)
threads, Win32 threads, DCE threads, DECthreads, or any of the draft
revisions of the pthreads standard. The trend is that most of these
systems are slowly adopting the pthreads standard API, with
application developers following suit to reduce porting woes.

Win32 does not, and is unlikely to ever, support pthreads
natively. This project seeks to provide a freely available and
high-quality solution to this problem.

%package -n mingw32-pthreads-static
Summary:        Static version of the MinGW Windows pthreads library
Requires:       mingw32-pthreads = %{version}-%{release}
Group:          Development/Libraries

%description -n mingw32-pthreads-static
Static version of the MinGW Windows pthreads library.

# Win64
%package -n mingw64-pthreads
Summary:        MinGW pthread library for the win64 target

# Fix upgrade path when upgrading from the testing repo to F17
Obsoletes:      mingw64-winpthreads < 0-0.2.20120224
Provides:       mingw64-winpthreads = 0-0.2.20120224

%description -n mingw64-pthreads
The POSIX 1003.1-2001 standard defines an application programming
interface (API) for writing multithreaded applications. This interface
is known more commonly as pthreads. A good number of modern operating
systems include a threading library of some kind: Solaris (UI)
threads, Win32 threads, DCE threads, DECthreads, or any of the draft
revisions of the pthreads standard. The trend is that most of these
systems are slowly adopting the pthreads standard API, with
application developers following suit to reduce porting woes.

Win32 does not, and is unlikely to ever, support pthreads
natively. This project seeks to provide a freely available and
high-quality solution to this problem.

%package -n mingw64-pthreads-static
Summary:        Static version of the MinGW Windows pthreads library
Requires:       mingw64-pthreads = %{version}-%{release}
Group:          Development/Libraries

%description -n mingw64-pthreads-static
Static version of the MinGW Windows pthreads library.


%?mingw_debug_package


%prep
%setup -q -n pthreads-w32-%{prerelease_date}

%patch0 -p1
%patch1 -p1
%patch2 -p1

# Duplicate all source code as the Makefile doesn't support out of source compilation
mkdir ../build_win32
cp -Rv * ../build_win32
mkdir ../build_win64
mv * ../build_win64
mv ../build_win32 ../build_win64 .

# But move the documentation files back to the main tree
pushd build_win32
mv ANNOUNCE BUGS ChangeLog CONTRIBUTORS COPYING COPYING.LIB ..
mv FAQ MAINTAINERS NEWS PROGRESS README README.NONPORTABLE TODO ..
popd

# Use the correct compilers
sed -i s/"target  = "/"target  = %{mingw32_target}"/ build_win32/GNUmakefile
sed -i s/"target  = "/"target  = %{mingw64_target}"/ build_win64/GNUmakefile


%build
pushd build_win32
make clean
make CROSS=%{mingw32_host}- GC-inlined
make clean
make CROSS=%{mingw32_host}- GCE-inlined
mv libpthreadGC2.a libpthreadGC2.dll.a
mv libpthreadGCE2.a libpthreadGCE2.dll.a
make clean
make CROSS=%{mingw32_host}- GC-static
popd

pushd build_win64
make clean
make CROSS=%{mingw64_host}- GC-inlined
make clean
make CROSS=%{mingw64_host}- GCE-inlined
mv libpthreadGC2.a libpthreadGC2.dll.a
mv libpthreadGCE2.a libpthreadGCE2.dll.a
make clean
make CROSS=%{mingw64_host}- GC-static
popd


%check
%if %{run_tests}
pushd build_win32/tests
make clean
make QAPC= CC=%{mingw32_cc} XXCFLAGS="-D__CLEANUP_C" TEST=GC all-pass
make clean
make QAPC= CC=%{mingw32_cc} XXCFLAGS="-D__CLEANUP_C" TEST=GCE all-pass
make clean
make QAPC= CC=%{mingw32_cc} XXCFLAGS="-D__CLEANUP_C" TEST=GC all-pass
make clean
make QAPC= CC=%{mingw32_cc} XXCFLAGS="-D__CLEANUP_C" TEST=GCE all-pass
popd

pushd build_win64/tests
make clean
make QAPC= CC=%{mingw64_cc} XXCFLAGS="-D__CLEANUP_C" TEST=GC all-pass
make clean
make QAPC= CC=%{mingw64_cc} XXCFLAGS="-D__CLEANUP_C" TEST=GCE all-pass
make clean
make QAPC= CC=%{mingw64_cc} XXCFLAGS="-D__CLEANUP_C" TEST=GC all-pass
make clean
make QAPC= CC=%{mingw64_cc} XXCFLAGS="-D__CLEANUP_C" TEST=GCE all-pass
popd
%endif


%install
mkdir -p $RPM_BUILD_ROOT%{mingw32_bindir}
mkdir -p $RPM_BUILD_ROOT%{mingw32_libdir}
mkdir -p $RPM_BUILD_ROOT%{mingw32_includedir}
mkdir -p $RPM_BUILD_ROOT%{mingw64_bindir}
mkdir -p $RPM_BUILD_ROOT%{mingw64_libdir}
mkdir -p $RPM_BUILD_ROOT%{mingw64_includedir}

install -m 0755 build_win32/*.dll $RPM_BUILD_ROOT%{mingw32_bindir}
install -m 0644 build_win32/*.a $RPM_BUILD_ROOT%{mingw32_libdir}
install -m 0644 build_win32/pthread.h $RPM_BUILD_ROOT%{mingw32_includedir}
install -m 0644 build_win32/sched.h $RPM_BUILD_ROOT%{mingw32_includedir}
install -m 0644 build_win32/semaphore.h $RPM_BUILD_ROOT%{mingw32_includedir}
install -m 0755 build_win64/*.dll $RPM_BUILD_ROOT%{mingw64_bindir}
install -m 0644 build_win64/*.a $RPM_BUILD_ROOT%{mingw64_libdir}
install -m 0644 build_win64/pthread.h $RPM_BUILD_ROOT%{mingw64_includedir}
install -m 0644 build_win64/sched.h $RPM_BUILD_ROOT%{mingw64_includedir}
install -m 0644 build_win64/semaphore.h $RPM_BUILD_ROOT%{mingw64_includedir}

# Create a symlink from libpthreadGC2.a to libpthread.a because of BZ #498616
ln -s libpthreadGC2.a $RPM_BUILD_ROOT%{mingw32_libdir}/libpthread.a
ln -s libpthreadGC2.dll.a $RPM_BUILD_ROOT%{mingw32_libdir}/libpthread.dll.a
ln -s libpthreadGC2.a $RPM_BUILD_ROOT%{mingw64_libdir}/libpthread.a
ln -s libpthreadGC2.dll.a $RPM_BUILD_ROOT%{mingw64_libdir}/libpthread.dll.a


%files -n mingw32-pthreads
%doc ANNOUNCE BUGS ChangeLog CONTRIBUTORS COPYING COPYING.LIB
%doc FAQ MAINTAINERS NEWS PROGRESS README README.NONPORTABLE TODO
%{mingw32_bindir}/pthreadGC2.dll
%{mingw32_bindir}/pthreadGCE2.dll
%{mingw32_libdir}/libpthread.dll.a
%{mingw32_libdir}/libpthreadGC2.dll.a
%{mingw32_libdir}/libpthreadGCE2.dll.a
%{mingw32_includedir}/*.h

%files -n mingw32-pthreads-static
%{mingw32_libdir}/libpthread.a
%{mingw32_libdir}/libpthreadGC2.a

%files -n mingw64-pthreads
%doc ANNOUNCE BUGS ChangeLog CONTRIBUTORS COPYING COPYING.LIB
%doc FAQ MAINTAINERS NEWS PROGRESS README README.NONPORTABLE TODO
%{mingw64_bindir}/pthreadGC2.dll
%{mingw64_bindir}/pthreadGCE2.dll
%{mingw64_libdir}/libpthread.dll.a
%{mingw64_libdir}/libpthreadGC2.dll.a
%{mingw64_libdir}/libpthreadGCE2.dll.a
%{mingw64_includedir}/*.h

%files -n mingw64-pthreads-static
%{mingw64_libdir}/libpthread.a
%{mingw64_libdir}/libpthreadGC2.a


%changelog
* Fri Jul 20 2012 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 2.8.0-22.20110511cvs
- Rebuilt for https://fedoraproject.org/wiki/Fedora_18_Mass_Rebuild

* Mon Jul  9 2012 Erik van Pienbroek <epienbro@fedoraproject.org> - 2.8.0-21.20110511cvs
- Added -static subpackage

* Fri Mar  9 2012 Erik van Pienbroek <epienbro@fedoraproject.org> - 2.8.0-20.20110511cvs
- Fix upgrade path when upgrading from the testing repo to F17

* Tue Mar  6 2012 Erik van Pienbroek <epienbro@fedoraproject.org> - 2.8.0-19.20110511cvs
- Made the package compliant with the new MinGW packaging guidelines
- Added win64 support

* Tue Mar 06 2012 Kalev Lember <kalevlember@gmail.com> - 2.8.0-18.20110511cvs
- Renamed the source package to mingw-pthreads (#800414)
- Use automatic dep extraction
- Use mingw macros without leading underscore

* Mon Feb 27 2012 Erik van Pienbroek <epienbro@fedoraproject.org> - 2.8.0-17.20110511cvs
- Rebuild against the mingw-w64 toolchain

* Fri Jan 13 2012 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 2.8.0-16.20110511cvs
- Rebuilt for https://fedoraproject.org/wiki/Fedora_17_Mass_Rebuild

* Thu May 12 2011 Kalev Lember <kalev@smartlink.ee> - 2.8.0-15.20110511cvs
- Update to 2011-05-11 cvs snapshot
- Dropped upstreamed mingw32-pthreads-2.8.0-w32.patch
- Don't install pthread.def
- Spec clean up from unnecessary cruft not needed in current Fedora releases

* Thu May 12 2011 Kalev Lember <kalev@smartlink.ee> - 2.8.0-14
- Don't install private headers, most notably config.h

* Fri Apr 22 2011 Kalev Lember <kalev@smartlink.ee> - 2.8.0-13
- Rebuilt for pseudo-reloc version mismatch (#698827)

* Tue Feb 08 2011 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 2.8.0-12
- Rebuilt for https://fedoraproject.org/wiki/Fedora_15_Mass_Rebuild

* Mon Nov 16 2009 Levente Farkas <lfarkas@lfarkas.org> - 2.8.0-11
- add comment about mingw32-pthreads-2.8.0-w32.patch

* Sun Nov 15 2009 Levente Farkas <lfarkas@lfarkas.org> - 2.8.0-10
- added mingw32-pthreads-2.8.0-w32.patch

* Sat Jul 25 2009 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 2.8.0-9
- Rebuilt for https://fedoraproject.org/wiki/Fedora_12_Mass_Rebuild

* Fri May 22 2009 Erik van Pienbroek <epienbro@fedoraproject.org> - 2.8.0-8
- Create a symlink from libpthreadGC2.a to libpthread.a because of BZ #498616

* Fri Mar 13 2009 Richard W.M. Jones <rjones@redhat.com> - 2.8.0-7
- Move header files to system include directory.

* Wed Feb 25 2009 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 2.8.0-6
- Rebuilt for https://fedoraproject.org/wiki/Fedora_11_Mass_Rebuild

* Fri Feb 20 2009 Richard W.M. Jones <rjones@redhat.com> - 2.8.0-5
- Rebuild for mingw32-gcc 4.4

* Tue Jan 13 2009 Richard W.M. Jones <rjones@redhat.com> - 2.8.0-4
- Cleanup to the spec file, no functional changes.

* Mon Dec 29 2008 Levente Farkas <lfarkas@lfarkas.org> - 2.8.0-3
- minor cleanup

* Fri Oct 10 2008 Richard W.M. Jones <rjones@redhat.com> - 2.8.0-2
- Initial RPM release.
